@@ -19,10 +19,8 @@
  */
 
 #include <config.h>
-
 #include <glib.h>
 #include <sys/types.h>
-#include <regex.h>
 #include <string.h>
 #include <time.h>
 #include <gtk/gtk.h>
@@ -32,8 +30,6 @@
 #include "gossip-utils.h"
 #include "gossip-chat-view.h"
 
-#define DINGUS "(((mailto|news|telnet|nttp|file|http|ftp|https|dav)://)|(www|ftp)[-A-Za-z0-9]*\\.)[-A-Za-z0-9\\.]+(:[0-9]*)?(/[-A-Za-z0-9_\\$\\.\\+\\!\\*\\(\\),;:@&=\\?/~\\#\\%]*[^]'\\.}>\\) ,\\\"])?"
-
 /* Number of seconds between timestamps when using normal mode, 5 minutes */
 #define TIMESTAMP_INTERVAL 300
 
@@ -99,6 +95,8 @@ static gint num_smileys = G_N_ELEMENTS (smileys);
 static void       chat_view_class_init      (GossipChatViewClass *klass);
 static void       chat_view_init            (GossipChatView      *view);
 static void       chat_view_finalize        (GObject             *object);
+static void       chat_view_size_allocate   (GtkWidget           *widget,
+					     GtkAllocation       *alloc);
 static void       chat_view_setup_tags      (GossipChatView      *view);
 static void       chat_view_populate_popup  (GossipChatView      *view,
 					     GtkMenu             *menu,
@@ -126,9 +124,6 @@ static void       chat_view_realize_cb      (GossipChatView      *widget,
 static void       chat_view_clear_view_cb   (GtkMenuItem *menuitem,
 					     GossipChatView *view);
 
-static gint       chat_view_url_regex_match (const gchar         *msg,
-					     GArray              *start,
-					     GArray              *end);
 static void       
 chat_view_insert_text_with_emoticons        (GtkTextBuffer       *buf,
 					     GtkTextIter         *iter, 
@@ -142,9 +137,11 @@ chat_view_maybe_append_timestamp             (GossipChatView *view,
 static void
 chat_view_maybe_append_datestamp             (GossipChatView *view);
 
+static gboolean chat_view_is_scrolled_down   (GossipChatView *view);
+
 
-static regex_t dingus;
 extern GConfClient *gconf_client;
+static GObjectClass *parent_class = NULL;
 
 GType
 gossip_chat_view_get_type (void)
@@ -175,9 +172,13 @@ gossip_chat_view_get_type (void)
 static void
 chat_view_class_init (GossipChatViewClass *klass)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GObjectClass   *object_class = G_OBJECT_CLASS (klass);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+
+	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = chat_view_finalize;
+	widget_class->size_allocate = chat_view_size_allocate;
 }
 
 static void
@@ -219,8 +220,26 @@ chat_view_finalize (GObject *object)
 	priv = view->priv;
 
 	g_object_unref (priv->buffer);
-	
+	g_date_free (priv->last_datestamp);
+	 
 	g_free (view->priv);
+
+	G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+static void
+chat_view_size_allocate (GtkWidget     *widget,
+			 GtkAllocation *alloc)
+{
+	gboolean down;
+	
+	down = chat_view_is_scrolled_down (GOSSIP_CHAT_VIEW (widget));
+		
+	GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, alloc);
+
+	if (down) {
+		gossip_chat_view_scroll_down (GOSSIP_CHAT_VIEW (widget));
+	}
 }
 
 static void
@@ -491,42 +510,6 @@ chat_view_clear_view_cb (GtkMenuItem *menuitem, GossipChatView *view)
 	gossip_chat_view_clear (view);
 }
 
-static gint
-chat_view_url_regex_match (const gchar *msg,
-			   GArray      *start,
-			   GArray      *end)
-{
-	static gboolean inited = FALSE;
-	regmatch_t      matches[1];
-	gint            ret = 0;
-	gint            num_matches = 0;
-	gint            offset = 0;
-
-	if (!inited) {
-		memset (&dingus, 0, sizeof (regex_t));
-		regcomp (&dingus, DINGUS, REG_EXTENDED);
-		inited = TRUE;
-	}
-
-	while (!ret) {
-		ret = regexec (&dingus, msg + offset, 1, matches, 0);
-
-		if (ret == 0) {
-			gint s;
-			
-			num_matches++;
-
-			s = matches[0].rm_so + offset;
-			offset = matches[0].rm_eo + offset;
-			
-			g_array_append_val (start, s);
-			g_array_append_val (end, offset);
-		}
-	}
-		
-	return num_matches;
-}
-
 static void
 chat_view_insert_text_with_emoticons (GtkTextBuffer *buf,
 				      GtkTextIter   *iter, 
@@ -715,11 +698,11 @@ chat_view_is_scrolled_down (GossipChatView *view)
 	GtkWidget *sw;
 
 	/* Turn off for version before 2.2.3. */
-	if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION == 2 &&
-	    GTK_MAJOR_VERSION < 3) {
+	if (GTK_MINOR_VERSION < 2 ||
+	    (GTK_MINOR_VERSION == 2 && GTK_MICRO_VERSION < 3)) {
 		return TRUE;
 	}
-	
+
 	sw = gtk_widget_get_parent (GTK_WIDGET (view));
 	if (GTK_IS_SCROLLED_WINDOW (sw)) {
 		GtkAdjustment *vadj;
@@ -734,6 +717,77 @@ chat_view_is_scrolled_down (GossipChatView *view)
 	return TRUE;
 }
 
+void
+gossip_chat_view_scroll_down (GossipChatView *view)
+{
+	GtkTextBuffer *buffer;
+	GtkTextIter    iter;
+	GtkTextMark   *mark;
+
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
+
+	gtk_text_buffer_get_end_iter (buffer, &iter);
+	mark = gtk_text_buffer_create_mark (buffer,
+					    NULL,
+					    &iter,
+					    FALSE);
+	
+	gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (view),
+				      mark,
+				      0.0,
+				      FALSE,
+				      0,
+				      0);
+}
+
+static gboolean
+chat_view_check_nick_highlight (const gchar *msg, const gchar *to)
+{
+	gboolean  ret_val;
+	gchar    *cf_msg, *cf_to;
+	gchar    *ch;
+
+	ret_val = FALSE;
+	
+	cf_msg = g_utf8_casefold (msg, -1);
+	cf_to = g_utf8_casefold (to, -1);
+
+	ch = strstr (cf_msg, cf_to);
+
+	if (ch == NULL) {
+		goto finished;
+	}
+		
+	if (ch != cf_msg) {
+		/* Not first in the message */
+		if ((*(ch - 1) != ' ') &&
+		    (*(ch - 1) != ',') &&
+		    (*(ch - 1) != '.')) {
+			goto finished;
+		}
+	}
+
+	ch = ch + g_utf8_strlen (cf_to, -1);
+	if (ch >= cf_msg + g_utf8_strlen (cf_msg, -1)) {
+		ret_val = TRUE;
+		goto finished;
+	}
+
+	if ((*ch == ' ') ||
+	    (*ch == ',') ||
+	    (*ch == '.') ||
+	    (*ch == ':')) {
+		ret_val = TRUE;
+		goto finished;
+	}
+
+finished:
+	g_free (cf_msg);
+	g_free (cf_to);
+
+	return ret_val;
+}
+
 void
 gossip_chat_view_append_chat_message (GossipChatView *view,
 				      const gchar    *time_str,
@@ -744,7 +798,6 @@ gossip_chat_view_append_chat_message (GossipChatView *view,
 	GtkTextBuffer *buffer;
 	GtkTextIter    iter;
 	gchar         *nick_tag;
-	GtkTextMark   *mark;
 	gint           num_matches, i;
 	GArray        *start, *end;
 	gboolean       bottom;
@@ -770,7 +823,7 @@ gossip_chat_view_append_chat_message (GossipChatView *view,
 			if (to && strcmp (from, to) == 0) {
 				nick_tag = "nick-me";
 			}
-			else if (to && strstr (msg, to)) {
+			else if (to && chat_view_check_nick_highlight (msg, to)) {
 				nick_tag = "nick-highlight";
 			} else {
 				nick_tag = "nick-other";
@@ -826,7 +879,7 @@ gossip_chat_view_append_chat_message (GossipChatView *view,
 	start = g_array_new (FALSE, FALSE, sizeof (gint));
 	end = g_array_new (FALSE, FALSE, sizeof (gint));
 	
-	num_matches = chat_view_url_regex_match (msg, start, end);
+	num_matches = gossip_utils_url_regex_match (msg, start, end);
 
 	if (num_matches == 0) {
 		gtk_text_buffer_get_end_iter (buffer, &iter);
@@ -876,8 +929,8 @@ gossip_chat_view_append_chat_message (GossipChatView *view,
 		}
 	}
 
-	g_array_free (start, FALSE);
-	g_array_free (end, FALSE);
+	g_array_free (start, TRUE);
+	g_array_free (end, TRUE);
 	
 	gtk_text_buffer_get_end_iter (buffer, &iter);
 	gtk_text_buffer_insert (buffer,
@@ -889,18 +942,7 @@ gossip_chat_view_append_chat_message (GossipChatView *view,
 	 * bottom before.
 	 */
 	if (bottom) {
-		gtk_text_buffer_get_end_iter (buffer, &iter);
-		mark = gtk_text_buffer_create_mark (buffer,
-						    NULL,
-						    &iter,
-						    FALSE);
-		
-		gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (view),
-					      mark,
-					      0.0,
-					      FALSE,
-					      0,
-					      0);
+		gossip_chat_view_scroll_down (view);
 	}
 }
 
@@ -913,7 +955,6 @@ gossip_chat_view_append_event_msg (GossipChatView *view,
 	GtkTextIter    iter;
 	gchar         *stamp;
 	gchar         *msg;
-	GtkTextMark   *mark;
 	gboolean       bottom;
 
 	bottom = chat_view_is_scrolled_down (view);
@@ -948,18 +989,7 @@ gossip_chat_view_append_event_msg (GossipChatView *view,
 				1);
 
 	if (bottom) {
-		gtk_text_buffer_get_end_iter (buffer, &iter);
-		mark = gtk_text_buffer_create_mark (buffer,
-						    NULL,
-						    &iter,
-						    FALSE);
-		
-		gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (view),
-					      mark,
-					      0.0,
-					      FALSE,
-					      0,
-					      0);
+		gossip_chat_view_scroll_down (view);
 	}
 }
 
